package org.eclipse.cdt.rpm.editor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.eclipse.cdt.rpm.editor.parser.Specfile;
import org.eclipse.cdt.rpm.editor.parser.SpecfileElement;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.Region;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;
import org.eclipse.jface.text.templates.DocumentTemplateContext;
import org.eclipse.jface.text.templates.Template;
import org.eclipse.jface.text.templates.TemplateContext;
import org.eclipse.jface.text.templates.TemplateContextType;
import org.eclipse.jface.text.templates.TemplateException;
import org.eclipse.jface.text.templates.TemplateProposal;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;

public class SpecfileCompletionProcessor implements IContentAssistProcessor {

	private static final String TEMPLATE_ICON = "icons/template.gif";

	private static final String MACRO_ICON = "icons/rpmmacro.gif";
	
	private static final String PACKAGE_ICON = "icons/rpm.gif";

	private static final String PREAMBLE_SECTION_TEMPLATE = "org.eclipse.cdt.rpm.editor.preambleSection";

	private static final String PRE_SECTION_TEMPLATE = "org.eclipse.cdt.rpm.editor.preSection";

	private static final String BUILD_SECTION_TEMPLATE = "org.eclipse.cdt.rpm.editor.buildSection";

	private static final String INSTALL_SECTION_TEMPLATE = "org.eclipse.cdt.rpm.editor.installSection";

	private static final String CHANGELOG_SECTION_TEMPLATE = "org.eclipse.cdt.rpm.editor.changelogSection";

	private static final class ProposalComparator implements Comparator {
		public int compare(Object o1, Object o2) {
			return ((TemplateProposal) o2).getRelevance()
					- ((TemplateProposal) o1).getRelevance();
		}
	}

	private static final Comparator proposalComparator = new ProposalComparator();
	private final SpecfileEditor sEditor;

	/**
	 * Default constructor
	 */
	public SpecfileCompletionProcessor(SpecfileEditor editor) {
		sEditor = editor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#computeCompletionProposals(org.eclipse.jface.text.ITextViewer,
	 *      int)
	 */
	public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer,
			int offset) {
		List result = new ArrayList();
		Specfile specfile = sEditor.getSpecfile();
		if (specfile == null)
			return null;
		ITextSelection selection = (ITextSelection) viewer
				.getSelectionProvider().getSelection();
		// adjust offset to start of normalized selection:
		if (selection.getOffset() != offset)
			offset = selection.getOffset();
		String prefix = getPrefix(viewer, offset);
		Region region = new Region(offset - prefix.length(), prefix.length()
				+ selection.getLength());
		ICompletionProposal[] templateProposals = computeTemplateProposals(
				viewer, region, specfile, prefix);
		ICompletionProposal[] rpmMacroProposals = computeRpmMacroProposals(
				viewer, region, prefix);
		ICompletionProposal[] rpmPackageProposals = computeRpmPackageProposals(
				viewer, region, specfile, prefix);		
		result.addAll(Arrays.asList(templateProposals));
		result.addAll(Arrays.asList(rpmMacroProposals));
		result.addAll(Arrays.asList(rpmPackageProposals));
		return (ICompletionProposal[]) result
				.toArray(new ICompletionProposal[result.size()]);
	}
	
	private ICompletionProposal[] computeTemplateProposals(ITextViewer viewer,
			IRegion region, Specfile specfile, String prefix) {
		TemplateContext context = createContext(viewer, region, specfile);
		if (context == null) {
			return new ICompletionProposal[0];
		}
		ITextSelection selection = (ITextSelection) viewer
				.getSelectionProvider().getSelection();
		context.setVariable("selection", selection.getText());
		String id = context.getContextType().getId();
		Template[] templates = Activator.getDefault().getTemplateStore()
				.getTemplates(id);
		List matches = new ArrayList();
		for (int i = 0; i < templates.length; i++) {
			Template template = templates[i];
			try {
				context.getContextType().validate(template.getPattern());
			} catch (TemplateException e) {
				continue;
			}
			int relevance = getRelevance(template, prefix);
			if (relevance > 0) {
				matches.add(new TemplateProposal(template, context, region,
						getImage(TEMPLATE_ICON), relevance));
			}
		}
		Collections.sort(matches, proposalComparator);
		return (ICompletionProposal[]) matches
				.toArray(new ICompletionProposal[matches.size()]);
	}

	/**
	 * Compute RPM macro proposals, these proposals are usable in the whole document.
	 * Return an array of RPM macro proposals for the given viewer, region, prefix.
	 * 
	 * @param viewer
	 *            the viewer for which the context is created
	 * @param region
	 *            the region into <code>document</code> for which the context
	 *            is created
	 * @param prefix
	 * 			  the prefix string to find
	 * @return 
	 *            a ICompletionProposal[]
	 */
	private ICompletionProposal[] computeRpmMacroProposals(ITextViewer viewer,
			IRegion region, String prefix) {
		List rpmMacroProposalsList = Activator.getDefault().getRpmMacroList().getProposals(prefix);
		if (rpmMacroProposalsList == null)
			return new ICompletionProposal[0];
		ArrayList proposals = new ArrayList();
		String[] item;
		Iterator iterator = rpmMacroProposalsList.iterator();
		while (iterator.hasNext()) {
			item = (String[]) iterator.next();
			proposals.add(new CompletionProposal("%{" + item[0].substring(1) + "}", 
							region.getOffset(), region.getLength(),
							item[0].length() + 2, getImage(MACRO_ICON),
							item[0], null, item[1]));
		}
		return (ICompletionProposal[]) proposals
				.toArray(new ICompletionProposal[proposals.size()]);
	}

	/**
	 * Compute RPM package proposals, these proposals are usable only in the preambule section.
	 * Return an array of RPM macro proposals for the given viewer, region, specfile, prefix.
	 * 
	 * @param viewer
	 *            the viewer for which the context is created
	 * @param region
	 *            the region into <code>document</code> for which the context
	 *            is created
	 * @param specfile
	 * 			  the specfile element
	 * @param prefix
	 * 			  the prefix string
	 * @return 
	 *            a ICompletionProposal[]
	 */
	private ICompletionProposal[] computeRpmPackageProposals(ITextViewer viewer,
			IRegion region, Specfile specfile, String prefix) {
		List rpmPkgProposalsList = Activator.getDefault().getRpmPackageList().getProposals(prefix);
		SpecfileElement[] elements = specfile.getSectionsElements();
		// Show rpm packages proposals only in the preambule section 
		if (region.getOffset() < elements[0].getLineEndPosition()) {
			if (rpmPkgProposalsList == null)
				return new ICompletionProposal[0];
			ArrayList proposals = new ArrayList();
			String[] item;
			Iterator iterator = rpmPkgProposalsList.iterator();
			while (iterator.hasNext()) {
				item = (String[]) iterator.next();
				proposals.add(new CompletionProposal(item[0], 
								region.getOffset(), region.getLength(),
								item[0].length(), getImage(PACKAGE_ICON),
								item[0], null, item[1]));
			}
			return (ICompletionProposal[]) proposals.toArray(new ICompletionProposal[proposals.size()]);
		} else {
			return new ICompletionProposal[0];	
		}
	}

	private TemplateContextType getContextType(Specfile specfile, int offset) {
		SpecfileElement[] elements = specfile.getSectionsElements();
		if (offset < elements[0].getLineEndPosition()) {
			return Activator.getDefault().getContextTypeRegistry()
					.getContextType(PREAMBLE_SECTION_TEMPLATE);
		} else if (offset < elements[1].getLineEndPosition()) {
			return Activator.getDefault().getContextTypeRegistry()
					.getContextType(PRE_SECTION_TEMPLATE);
		} else if (offset < elements[2].getLineEndPosition()) {
			return Activator.getDefault().getContextTypeRegistry()
					.getContextType(BUILD_SECTION_TEMPLATE);
		} else if (offset < elements[3].getLineEndPosition()) {
			return Activator.getDefault().getContextTypeRegistry()
					.getContextType(INSTALL_SECTION_TEMPLATE);
		} else {
			return Activator.getDefault().getContextTypeRegistry()
					.getContextType(CHANGELOG_SECTION_TEMPLATE);
		}

	}

	private TemplateContext createContext(ITextViewer viewer, IRegion region,
			Specfile specfile) {
		TemplateContextType contextType = getContextType(specfile, region
				.getOffset());
		if (contextType != null) {
			IDocument document = viewer.getDocument();
			return new DocumentTemplateContext(contextType, document, region
					.getOffset(), region.getLength());
		}
		return null;
	}
	
	private int getRelevance(Template template, String prefix) {
		if (template.getName().startsWith(prefix))
			return 90;
		return 0;
	}

	private String getPrefix(ITextViewer viewer, int offset) {
		int i = offset;
		IDocument document = viewer.getDocument();
		if (i > document.getLength())
			return "";

		try {
			while (i > 0) {
				char ch = document.getChar(i - 1);
				if (!Character.isLetterOrDigit(ch) && (ch != '%') && (ch != '_')&& (ch != '-'))
					break;
				i--;
			}
			return document.get(i, offset - i);
		} catch (BadLocationException e) {
			return "";
		}
	}

	private Image getImage(String imageRelativePath) {
		ImageRegistry registry = Activator.getDefault().getImageRegistry();
		Image image = registry.get(imageRelativePath);
		if (image == null) {
			ImageDescriptor desc = AbstractUIPlugin.imageDescriptorFromPlugin(
					"org.eclipse.cdt.rpm.editor", imageRelativePath);
			registry.put(imageRelativePath, desc);
			image = registry.get(imageRelativePath);
		}
		return image;
	}

	public IContextInformation[] computeContextInformation(ITextViewer viewer,
			int offset) {
		return null;
	}

	public char[] getCompletionProposalAutoActivationCharacters() {
		return null;
	}

	public char[] getContextInformationAutoActivationCharacters() {
		return null;
	}

	public String getErrorMessage() {
		return null;
	}

	public IContextInformationValidator getContextInformationValidator() {
		return null;
	}
}
