package org.eclipse.cdt.rpm.editor;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.cdt.rpm.editor.outline.SpecfileContentOutlinePage;
import org.eclipse.cdt.rpm.editor.parser.Specfile;
import org.eclipse.cdt.rpm.editor.parser.SpecfileParser;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.projection.ProjectionSupport;
import org.eclipse.jface.text.source.projection.ProjectionViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.texteditor.ContentAssistAction;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

public class SpecfileEditor extends TextEditor {

	private ColorManager colorManager;
	private SpecfileContentOutlinePage outlinePage;
	private IEditorInput input;
	private Specfile specfile;
	private ProjectionSupport projectionSupport;

	public SpecfileEditor() {
		super();
		colorManager = new ColorManager();
		setSourceViewerConfiguration(new SpecfileConfiguration(colorManager, this));
		setDocumentProvider(new SpecfileDocumentProvider());
	}
	public void dispose() {
		colorManager.dispose();
		super.dispose();
	}

	protected void doSetInput(IEditorInput newInput) throws CoreException
	{
		super.doSetInput(newInput);
		this.input = newInput;

		if (outlinePage != null)
			outlinePage.setInput(input);

		validateAndMark();
	}

	protected void editorSaved()
	{
		super.editorSaved();
		
		//we validate and mark document here
		validateAndMark();

		if (outlinePage != null)
			outlinePage.update();	
	}

	protected void validateAndMark()
	{
		try
		{
			IDocument document = getInputDocument();
			SpecfileErrorHandler specfileErrorHandler = new SpecfileErrorHandler(getInputFile(), document);
			specfileErrorHandler.removeExistingMarkers();

			SpecfileParser parser = new SpecfileParser();
			parser.setErrorHandler(specfileErrorHandler);
			specfile = parser.parse(document);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	protected IFile getInputFile()
	{
		IFileEditorInput ife = (IFileEditorInput) input;
		IFile file = ife.getFile();
		return file;
	}

	protected IDocument getInputDocument()
	{
		IDocument document = getDocumentProvider().getDocument(input);
		return document;
	}

	public Object getAdapter(Class required) {
		if (IContentOutlinePage.class.equals(required)) {
			if (outlinePage == null) {
				outlinePage= new SpecfileContentOutlinePage(this);
				if (getEditorInput() != null)
					outlinePage.setInput(getEditorInput());
			}
			return outlinePage;
		}
		if (projectionSupport != null) {
			Object adapter= projectionSupport.getAdapter(getSourceViewer(), required);
			if (adapter != null)
				return adapter;
		}
		return super.getAdapter(required);
	}
	
	public Specfile getSpecfile() {
		return specfile;
	}

	
	/*
	 * @see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#createSourceViewer(org.eclipse.swt.widgets.Composite, org.eclipse.jface.text.source.IVerticalRuler, int)
	 */
	protected ISourceViewer createSourceViewer(Composite parent, IVerticalRuler ruler, int styles) {
		fAnnotationAccess = createAnnotationAccess();
		fOverviewRuler = createOverviewRuler(getSharedColors());
		ISourceViewer viewer= new ProjectionViewer(parent, ruler, fOverviewRuler, true, styles);
		getSourceViewerDecorationSupport(viewer);
		return viewer;
	}

	/*
	 * @see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		super.createPartControl(parent);
		ProjectionViewer projectionViewer = (ProjectionViewer) getSourceViewer();
		projectionSupport = new ProjectionSupport(projectionViewer, getAnnotationAccess(), getSharedColors());
		projectionSupport.install();
		projectionViewer.doOperation(ProjectionViewer.TOGGLE);
	}
	
	/*
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#createActions()
	 */
	protected void createActions() {
		super.createActions();

		IAction action= new ContentAssistAction(
				getResourceBundle(),
				"ContentAssistProposal.",
				this);
		action.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS);
		setAction("ContentAssist", action);
		markAsStateDependentAction("ContentAssist", true);
	}
	
	
	// ContentAssistAction take a ResourceBundle but Resource bundles are not yet implemented on  
	// a plugin level, so we have add this method here as a quick fix.
	public ResourceBundle getResourceBundle() {
		ResourceBundle resourceBundle;
		try {
			resourceBundle= ResourceBundle.getBundle("org.eclipse.cdt.rpm.editor.SpecfileEditorMessages");
		} catch (MissingResourceException x) {
			resourceBundle= null;
		}
		return resourceBundle;
	}

}
