package org.eclipse.cdt.rpm.editor.preferences;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.cdt.rpm.editor.Activator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

public class RpmProposalsPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	public RpmProposalsPreferencePage() {
		super(GRID);
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		IntegerFieldEditor maxProposalsFieldEditor = new IntegerFieldEditor(PreferenceConstants.P_RPM_LIST_MAX_PROPOSALS, "Maximum proposals RPM info:", getFieldEditorParent());
		maxProposalsFieldEditor.setValidRange(1, 40);
		maxProposalsFieldEditor.setErrorMessage("Proposals informations limit must be an integer between 1 and 40");
		addField(maxProposalsFieldEditor);
		addField(
			new StringFieldEditor(PreferenceConstants.P_RPM_LIST_CMD, "Packages list command:", getFieldEditorParent()));
		addField(new StringFieldEditor(PreferenceConstants.P_RPM_LIST_FILEPATH, "Packages list path:", getFieldEditorParent()));
	}
	
    protected Control createContents(Composite parent) {
    	Composite fieldEditorComposite = (Composite) super.createContents(parent);
    	Button builRpmProposalsButton = new Button(fieldEditorComposite, SWT.PUSH); 
    	builRpmProposalsButton.setText("Build RPM packages proposals");
    	builRpmProposalsButton.addListener(SWT.Selection, new Listener(){

			public void handleEvent(Event event) {
		        IRunnableWithProgress runnable = new IRunnableWithProgress() {
		        	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						String rpmListCmd = getPreferenceStore().getString(PreferenceConstants.P_RPM_LIST_CMD);
						String rpmListFilepath =  getPreferenceStore().getString(PreferenceConstants.P_RPM_LIST_FILEPATH);
						try {
							String[] cmd = new String[] {"/bin/sh","-c", rpmListCmd};
							monitor.beginTask("Get RPM proposals list...", 100);
							monitor.worked(10);
							Process child = Runtime.getRuntime().exec(cmd);
							InputStream in = child.getInputStream();
							OutputStream out = new FileOutputStream(rpmListFilepath);
							monitor.setTaskName("Read RPM proposals from command " + rpmListCmd + " output stream...");
							monitor.worked(15);
							byte[] buf = new byte[1024];
							int len;
							int nbrOfByte = 15;
							while ((len = in.read(buf)) > 0) {
								monitor.setTaskName("Write RPM proposals list into " + rpmListFilepath + " file...");
								out.write(buf, 0, len);
								monitor.worked(nbrOfByte);
								nbrOfByte++;
							}
							in.close();
							out.close();
						} catch (IOException e) {
							e.printStackTrace();
						}
					}
		        };
		        try {
			 		ProgressMonitorDialog progressMonitor = new ProgressMonitorDialog(getShell());
			 		progressMonitor.run(true, false, runnable);
		        } catch(Exception e) {
			 		e.printStackTrace();
		        }
			}
    		
    	});
        return fieldEditorComposite;
    }

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		
	}
	
    

}